import logging
from sys import stderr
from signal import signal, SIGTERM, SIGINT

from tornado.ioloop import IOLoop

from tfw.event_handlers import EventHandlerBase
from tfw.builtins import PipeIOEventHandler
from tfw.config import TFWENV
from tfw.logging import Log, Logger, LogFormatter, VerboseLogFormatter

from pipe_io_auxlib import (
        SignMessagePipeIOEventHandler, VerifyMessagePipeIOEventHandler,
        BotPipeIOEventHandler,
        DeployPipeIOEventHandler, IdePipeIOEventHandler,
        FSMPipeIOEventHandler
)

LOG = logging.getLogger(__name__)


if __name__ == '__main__':
    Logger([
        Log(stderr, LogFormatter(20)),
        Log(TFWENV.LOGFILE, VerboseLogFormatter())
    ]).start()

    """
    Creates general purpose pipes.
    The first parameter associates the receiving pipe with a key, which is
    an empty string in this case. It has a special meaning, you can
    subscribe to every kind of message with this key.
    If you wish to filter incoming data, specify a single or more keys in
    a list, eg.: processmanager, ide, key...
    You can send/receive JSON messages to/from the TFW server as any user,
    because we gave read+write permissions, without that parameter, only
    the owner has access to the pipes.
    """
    json_pipe = PipeIOEventHandler(
        '',
        '/tmp/tfw_json_send',
        '/tmp/tfw_json_recv'
    )

    sign_pipe = SignMessagePipeIOEventHandler(
        '/tmp/tfw_sign_send',
        '/tmp/tfw_sign_recv',
        forwarding=True
    )

    verify_pipe = VerifyMessagePipeIOEventHandler(
        '/tmp/tfw_verify_send',
        '/tmp/tfw_verify_recv'
    )

    bot_pipe = BotPipeIOEventHandler(
        '/tmp/tfw_bot_send',
        '/tmp/tfw_bot_recv',
        permissions=0o666
    )

    deploy_pipe = DeployPipeIOEventHandler(
        '/tmp/tfw_deploy_send',
        '/tmp/tfw_deploy_recv',
        'webservice'
    )

    ide_pipe = IdePipeIOEventHandler(
        '/tmp/tfw_ide_send',
        '/tmp/tfw_ide_recv',
        'user_ops.py',
        selected=True
    )

    fsm_pipe = FSMPipeIOEventHandler(
        '/tmp/tfw_fsm_send',
        '/tmp/tfw_fsm_recv'
    )

    event_handlers = EventHandlerBase.get_local_instances()
    def stop(sig, frame):  # pylint: disable=unused-argument
        for eh in event_handlers:
            eh.stop()
        exit(0)
    signal(SIGTERM, stop)
    signal(SIGINT, stop)

    IOLoop.instance().start()
